unit NViewLib; {$D-}

interface

uses
  {$IFDEF WIN32}
  Windows,
  {$ELSE}
  WinTypes, WinProcs,
  {$ENDIF}
  Graphics;

type
  TNVLType = (nvlOther, nvlJPG, nvlGIF,
    nvlJIF, nvlBMP, nvlDIB, nvlRLE, nvlTGA, nvlPCX);

function NVLGetType(FileName: string): TNVLType;
function NVLGetWidth: Integer;
function NVLGetHeight: Integer;
function NVLLoadImage(const FileName: string; ShowProgress: Boolean): TBitmap;
function NVLSaveAsJPG(Quality: Integer; const FileName: string;
  Pic: TPicture): Boolean;

implementation

uses
  Classes, SysUtils;

{ A simple function to get FileName types }

function NVLGetType(FileName: string): TNVLType;
var  i: Byte;
begin
  Result := nvlOther;
  FileName := ExtractFileName(FileName);
  repeat
    i := Pos('.', FileName);
    if (i <> 0) then
      FileName := Copy(FileName, i+1, Length(FileName)-i);
  until (i = 0);
  for i := 1 to Length(FileName) do FileName[i] := UpCase(FileName[i]);
  if (FileName = 'JPG') then Result := nvlJPG else
    if (FileName = 'GIF') then Result := nvlGIF else
      if (FileName = 'JIF') then Result := nvlJIF else
        if (FileName = 'BMP') then Result := nvlBMP else
          if (FileName = 'DIB') then Result := nvlDIB else
            if (FileName = 'RLE') then Result := nvlRLE else
              if (FileName = 'TGA') then Result := nvlTGA else
                if (FileName = 'PCX') then Result := nvlPCX;
end;

{ Inform the user that the DLL is not loaded }

procedure DefaultWarning; far;
begin
  {$IFDEF WIN32}
  MessageBox(0, 'A required .DLL file, NViewLib.dll, was not found.',
    'Error Starting NViewLib.dll',
    MB_TASKMODAL or MB_ICONEXCLAMATION or MB_OK);
  {$ELSE}
  MessageBox(0, 'A required .DLL file, NVIEWL16.DLL, was not found.',
    'Error Starting NVIEWL16.DLL',
    MB_TASKMODAL or MB_ICONEXCLAMATION or MB_OK);
  {$ENDIF}
end;

function DefaultGet: Integer;
far; {$IFDEF WIN32} stdcall; {$ENDIF}
begin  DefaultWarning;  Result := 0;  end;

function DefaultLoad(FileName: PChar; ShowProgress: Boolean): HBitmap;
far; {$IFDEF WIN32} stdcall; {$ENDIF}
begin  DefaultWarning;  Result := 0;  end;

function DefaultSave(Quality: Integer; FileName: PChar): Boolean;
far; {$IFDEF WIN32} stdcall; {$ENDIF}
begin  DefaultWarning;  Result := False;  end;

function DefaultLanguage(Lang: PChar): Boolean;
far; {$IFDEF WIN32} stdcall; {$ENDIF}
begin  DefaultWarning;  Result := False;  end;

procedure DefaultCustomLang(pProgress, pError, pLoad, pErrLoad, pWarning: PChar);
far; {$IFDEF WIN32} stdcall; {$ENDIF}
begin  DefaultWarning;  end;

{ Virtual functions }

type
  TGetFunc = function: Integer;
  {$IFDEF WIN32} stdcall; {$ENDIF}
  TLoadFunc = function(FileName: PChar; ShowProgress: Boolean): HBitmap;
  {$IFDEF WIN32} stdcall; {$ENDIF}
  TSaveFunc = function(Quality: Integer; FileName: PChar): Boolean;
  {$IFDEF WIN32} stdcall; {$ENDIF}
  TLanguageFunc = function(Lang: PChar): Boolean;
  {$IFDEF WIN32} stdcall; {$ENDIF}
  TCustomLangProc = procedure(pProgress, pError, pLoad,
    pErrLoad, pWarning: PChar);
  {$IFDEF WIN32} stdcall; {$ENDIF}

const GetWidth: TGetFunc = DefaultGet;
const GetHeight: TGetFunc = DefaultGet;
const LoadImage: TLoadFunc = DefaultLoad;
const SaveJPG: TSaveFunc = DefaultSave;
const SetLanguage: TLanguageFunc = DefaultLanguage;
const SetCustomLang: TCustomLangProc = DefaultCustomLang;

{ Initialization procedures }

const
  NViewLibHandle: THANDLE = 0;

procedure DoneNViewLib; far;
begin
  if (NViewLibHandle >= 32) then FreeLibrary(NViewLibHandle);
  NViewLibHandle := 0;
end;

procedure InitNViewLib;
var  ErrMode: Word;
begin
  if (NViewLibHandle = 0) then
  begin
    ErrMode := SetErrorMode(SEM_NOOPENFILEERRORBOX);
    {$IFDEF WIN32}
    NViewLibHandle := LoadLibrary('NViewLib.dll');
    {$ELSE}
    NViewLibHandle := LoadLibrary('NVIEWL16.DLL');
    {$ENDIF}
    SetErrorMode(ErrMode);
    if (NViewLibHandle < 32) then NViewLibHandle := 0;
    if (NViewLibHandle >= 32) then
    begin
      {$IFDEF WIN32}
      GetWidth := GetProcAddress(NViewLibHandle, 'GetWidth');
      GetHeight := GetProcAddress(NViewLibHandle, 'GetHeight');
      LoadImage := GetProcAddress(NViewLibHandle, 'NViewLibLoad');
      SaveJPG := GetProcAddress(NViewLibHandle, 'NViewLibSaveAsJPG');
      SetLanguage := GetProcAddress(NViewLibHandle, 'NViewLibSetLanguage');
      SetCustomLang := GetProcAddress(NViewLibHandle, 'NViewLibSetCustomLang');
      {$ELSE}
      @GetWidth := GetProcAddress(NViewLibHandle, 'GetWidth');
      @GetHeight := GetProcAddress(NViewLibHandle, 'GetHeight');
      @LoadImage := GetProcAddress(NViewLibHandle, 'NViewLibLoad');
      @SaveJPG := GetProcAddress(NViewLibHandle, 'NViewLibSaveAsJPG');
      @SetLanguage := GetProcAddress(NViewLibHandle, 'NViewLibSetLanguage');
      @SetCustomLang := GetProcAddress(NViewLibHandle, 'NViewLibSetCustomLang');
      {$ENDIF}
    end;
    AddExitProc(DoneNViewLib);
  end;
end;

{ Unit functions }

function NVLGetWidth: Integer;
begin  InitNViewLib;  Result := GetWidth;  end;

function NVLGetHeight: Integer;
begin  InitNViewLib;  Result := GetHeight;  end;

function LoadFile(const FileName: string; SP: Boolean): TBitmap;
var
  FN: PChar;
  BMP: Graphics.TBitmap;
  R: TRect;
begin
  Result := nil;
  if (FileName = '') then Exit;
  InitNViewLib;
  GetMem(FN, Length(FileName)+1);
  StrPCopy(FN, FileName);
  { Use a temp bmp because NViewLib maintains only 1 handle! }
  BMP := Graphics.TBitmap.Create;
  try
    BMP.Handle := LoadImage(FN, SP);
  except
    raise;  BMP.Free;  Exit;
  end;
  FreeMem(FN, Length(FileName)+1);

  if (BMP.Handle <> 0) then
  begin
    Result := Graphics.TBitmap.Create;
    try
      with Result do
      begin
        Width := BMP.Width;  Height := BMP.Height;
        R := Rect(0, 0, Width, Height);
        Canvas.CopyRect(R, BMP.Canvas, R);
      end
    finally
      DeleteObject(BMP.Handle);
    end;
  end;
  BMP.Free;
end;

function SaveAsJPG(Quality: Integer; const FileName: string;
  Pic: TPicture): Boolean;
var
  Temp: string;
  FN: PChar;
  H: HBITMAP;
begin
  Result := False;
  if (FileName = '') then Exit;
  InitNViewLib;
  Temp := ExtractFilePath(FileName) + '$N$VIEW$.BMP';
  try
    Pic.SaveToFile(Temp);
  except
    raise;
    Exit;  {Oops - cannot save BMP! }
  end;
  GetMem(FN, Length(Temp)+1);
  StrPCopy(FN, Temp);
  try
    H := LoadImage(FN, False);
  except
    H := 0;
  end;
  FreeMem(FN, Length(Temp)+1);
  if (H = 0) then Exit;

  GetMem(FN, Length(FileName)+1);
  StrPCopy(FN, FileName);
  try
    Result := SaveJPG(Quality, FN);
  except
    Result := False;
  end;

  FreeMem(FN, Length(FileName)+1);
  DeleteObject(H);  { Remove the NViewLib Handle }
  DeleteFile(Temp);  { Delete the temp file }
end;

function NVLLoadImage(const FileName: string; ShowProgress: Boolean): TBitmap;
begin
  Result := LoadFile(FileName, ShowProgress);
end;

function NVLSaveAsJPG(Quality: Integer; const FileName: string;
  Pic: TPicture): Boolean;
begin
  Result := SaveAsJPG(Quality, FileName, Pic);
end;

end.
