unit PictEd16;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Dialogs,
  Buttons, DsgnIntf, StdCtrls, ExtCtrls, FileCtrl;

type
  TPictureEditorDlg2 = class(TForm)
    Save: TButton;
    SaveDialog: TSaveDialog;
    Clear: TButton;
    Cancel: TBitBtn;
    OK: TBitBtn;
    Shape1: TShape;
    Shape2: TShape;
    HelpBtn: TBitBtn;
    DriveComboBox: TDriveComboBox;
    DirectoryListBox: TDirectoryListBox;
    LabelDirectory: TLabel;
    EditFileName: TEdit;
    FileListBox: TFileListBox;
    FilterComboBox: TFilterComboBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Image1: TImage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SaveClick(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure ClearClick(Sender: TObject);
    procedure EditFileNameChange(Sender: TObject);
    procedure FileListBoxDblClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
  private
    Pic: TPicture;
  end;

  TPictureEditor2 = class(TComponent)
  private
    FGraphicClass: TGraphicClass;
    FPicture: TPicture;
    FPicDlg: TPictureEditorDlg2;
    procedure SetPicture(Value: TPicture);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: Boolean;
    property GraphicClass: TGraphicClass read FGraphicClass write FGraphicClass;
    property Picture: TPicture read FPicture write SetPicture;
  end;

{ TPictureProperty
  Property editor the TPicture properties (e.g. the Picture property).  Brings
  up a file open dialog allowing loading a picture file. }

  TPictureProperty2 = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
  end;

{ TGraphicProperty }

  TGraphicProperty2 = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
  end;

{ TGraphicEditor }

  TGraphicEditor2 = class(TDefaultEditor)
  public
    procedure EditProperty(PropertyEditor: TPropertyEditor;
      var Continue, FreeEditor: Boolean); override;
  end;

implementation

uses TypInfo, SysUtils, LibConst, LibHelp, NViewLib;

{$R *.DFM}

{ TPictureEditorDlg }

procedure TPictureEditorDlg2.FormCreate(Sender: TObject);
begin
  HelpContext := hcDPictureEditor;
  Pic := TPicture.Create;
  Save.Enabled := False;
end;

procedure TPictureEditorDlg2.FormDestroy(Sender: TObject);
begin
  Pic.Free;
end;

procedure TPictureEditorDlg2.SaveClick(Sender: TObject);
begin
  if Pic.Graphic <> nil then
    with SaveDialog do
    begin
      DefaultExt := GraphicExtension(TGraphicClass(Pic.Graphic.ClassType));
      Filter := GraphicFilter(TGraphicClass(Pic.Graphic.ClassType));
      if Execute then Pic.SaveToFile(Filename);
    end;
end;

procedure TPictureEditorDlg2.FormPaint(Sender: TObject);
var
  DrawRect: TRect;
  SNone: string;
begin
  Canvas.Brush.Color := Color;
  with Shape1 do
    DrawRect := Rect(Left, Top, Left + Width, Top + Height);
  if Pic.Width > 0 then
  begin
    with DrawRect do
      if (Pic.Width > Right - Left) or (Pic.Height > Bottom - Top) then
      begin
        if Pic.Width > Pic.Height then
          Bottom := Top + MulDiv(Pic.Height, Right - Left, Pic.Width)
        else
          Right := Left + MulDiv(Pic.Width, Bottom - Top, Pic.Height);
        Canvas.StretchDraw(DrawRect, Pic.Graphic);
      end
      else
        with DrawRect do
          Canvas.Draw(Left + (Right - Left - Pic.Width) div 2, Top + (Bottom - Top -
            Pic.Height) div 2, Pic.Graphic);
  end
  else
    with DrawRect, Canvas do
    begin
      SNone := LoadStr(srNone);
      TextOut(Left + (Right - Left - TextWidth(SNone)) div 2, Top + (Bottom -
        Top - TextHeight(SNone)) div 2, SNone);
    end;
end;

procedure TPictureEditorDlg2.ClearClick(Sender: TObject);
begin
  Pic.Graphic := nil;
  Invalidate;
  Save.Enabled := False;
  Clear.Enabled := False;
end;

{ TPictureEditor }

constructor TPictureEditor2.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPicture := TPicture.Create;
  FPicDlg := TPictureEditorDlg2.Create(Self);
  FGraphicClass := TGraphic;
end;

destructor TPictureEditor2.Destroy;
begin
  FPicture.Free;
  inherited Destroy;
end;

function TPictureEditor2.Execute: Boolean;
begin
  FPicDlg.Pic.Assign(FPicture);
  {HelpContext := hcDLoadPicture;}
  with FPicDlg.FilterComboBox do
  begin
    Filter := GraphicFilter(GraphicClass);
    if (GraphicClass = TGraphic) or (GraphicClass = TBitmap) then
      Filter := 'All (Supported/Imported)|'
        + '*.bmp;*.ico;*.emf;*.wmf;*.jpg;*.jif;*.gif;*.dib;*.rle;*.tga;*.pcx|'
        + Filter
        + '|Import JPeg''s (*.jpg;*.jif)|*.jpg;*.jif'
        + '|Import GIF''s (*.gif)|*.gif'
        + '|Import DIB''s (*.dib)|*.dib'
        + '|Import RLE''s (*.rle)|*.rle'
        + '|Import TGA''s (*.tga)|*.tga'
        + '|Import PCX''s (*.pcx)|*.pcx';
  end;
  with FPicDlg.SaveDialog do
  begin
    Options := [ofHideReadOnly, ofFileMustExist, ofShowHelp];
    DefaultExt := GraphicExtension(GraphicClass);
    Filter := GraphicFilter(GraphicClass);
    HelpContext := hcDSavePicture;
  end;
  FPicDlg.Save.Enabled := FPicture.Graphic <> nil;
  FPicDlg.Clear.Enabled := FPicture.Graphic <> nil;
  Result := FPicDlg.ShowModal = mrOK;
  if Result then FPicture.Assign(FPicDlg.Pic);
end;

procedure TPictureEditor2.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
end;

{ TPictureProperty }

procedure TPictureProperty2.Edit;
var
  PictureEditor: TPictureEditor2;
begin
  PictureEditor := TPictureEditor2.Create(nil);
  try
    PictureEditor.Picture := TPicture(Pointer(GetOrdValue));
    if PictureEditor.Execute then
      SetOrdValue(Longint(PictureEditor.Picture));
  finally
    PictureEditor.Free;
  end;
end;

function TPictureProperty2.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

function TPictureProperty2.GetValue: string;
var
  Picture: TPicture;
begin
  Picture := TPicture(GetOrdValue);
  if Picture.Graphic = nil then
    Result := LoadStr(srNone) else
    Result := '(' + Picture.Graphic.ClassName + ')';
end;

procedure TPictureProperty2.SetValue(const Value: string);
begin
  if Value = '' then SetOrdValue(0);
end;

{ TGraphicProperty }

procedure TGraphicProperty2.Edit;
var
  PictureEditor: TPictureEditor2;
begin
  PictureEditor := TPictureEditor2.Create(nil);
  try
    PictureEditor.GraphicClass := TGraphicClass(GetTypeData(GetPropType)^.ClassType);
    PictureEditor.Picture.Graphic := TGraphic(Pointer(GetOrdValue));
    if PictureEditor.Execute then
      if (PictureEditor.Picture.Graphic = nil) or
         (PictureEditor.Picture.Graphic is PictureEditor.GraphicClass) then
        SetOrdValue(LongInt(PictureEditor.Picture.Graphic))
      else
        raise Exception.Create(LoadStr(SInvalidFormat));
  finally
    PictureEditor.Free;
  end;
end;

function TGraphicProperty2.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

function TGraphicProperty2.GetValue: string;
var
  Graphic: TGraphic;
begin
  Graphic := TGraphic(GetOrdValue);
  if (Graphic = nil) or Graphic.Empty then
    Result := LoadStr(srNone) else
    Result := '(' + Graphic.ClassName + ')';
end;

procedure TGraphicProperty2.SetValue(const Value: string);
begin
  if Value = '' then SetOrdValue(0);
end;

{ TPictureEditor }

procedure TGraphicEditor2.EditProperty(PropertyEditor: TPropertyEditor;
  var Continue, FreeEditor: Boolean);
var
  PropName: string;
begin
  PropName := PropertyEditor.GetName;
  if (CompareText(PropName, 'PICTURE') = 0) or
    (CompareText(PropName, 'IMAGE') = 0) then
  begin
    PropertyEditor.Edit;
    Continue := False;
  end;
end;

procedure TPictureEditorDlg2.EditFileNameChange(Sender: TObject);
var  FT: TNVLType;
begin
  if (FileExists(EditFileName.Text)) and (Pos('*', EditFileName.Text) = 0) then
  begin
    FT := NVLGetType(EditFileName.Text);
    if (FT <> nvlOther) and (FT <> nvlBMP) then
      Pic.Bitmap := NVLLoadImage(EditFileName.Text, True)
    else
      Pic.LoadFromFile(EditFileName.Text);
    Invalidate;
    Save.Enabled := (Pic.Graphic <> nil);
    Clear.Enabled := (Pic.Graphic <> nil);
  end;
end;

procedure TPictureEditorDlg2.FileListBoxDblClick(Sender: TObject);
begin
  if OK.Enabled then ModalResult := mrOk;
end;

procedure TPictureEditorDlg2.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(HelpContext);
end;

end.
